//*****************************************************************************
//                                PrcBase.hpp                                 *
//                               -------------                                *
//  Description : Provides a wrapper for the wxProcess class adding a bit of  *
//                desirable functionality.                                    *
//  Started     : 29/01/2004                                                  *
//  Last Update : 15/02/2006                                                  *
//  Copyright   : (C) 2004 by MSWaters                                        *
//  Email       : M.Waters@bom.gov.au                                         *
//*****************************************************************************

//*****************************************************************************
//                                                                            *
//    This program is free software; you can redistribute it and/or modify    *
//    it under the terms of the GNU General Public License as published by    *
//    the Free Software Foundation; either version 2 of the License, or       *
//    (at your option) any later version.                                     *
//                                                                            *
//*****************************************************************************

#ifndef PRCBASE_HPP
#define PRCBASE_HPP

// System Includes

#include <iostream>

// wxWindows Includes

#include <wx/wx.h>
#include <wx/filename.h>
#include <wx/process.h>
#include <wx/utils.h>

// Application Includes

#include "TextCtrl.hpp"

// Local Constant Declarations

#define  DEFLOGFILE  wxT("./prcbase.log")  // Default log file path and name

//*****************************************************************************

class PrcBase : public wxProcess
{
  protected:

    wxFileName  m_oFnBinary;  // The file name of the binary to be executed
    wxString    m_osArgLst;   // The argument list appended to the binary name
    int         m_iPid;       // The process ID
    wxFileName  m_oFnLog;     // The file name of the simulation log
    wxString    m_osErrMsg;   // Error message

    bool  bFindBinary( wxFileName & oFnBinary );
    bool  bLogOutput ( void );

  public:

    PrcBase( int iFlags=wxPROCESS_REDIRECT );
    ~PrcBase( );

    bool  bBinExists( void );

    bool  bIsExec( void ) { return( m_iPid>0 ? TRUE : FALSE ); }
    bool  bIsOk  ( void ) { return( m_osErrMsg.IsEmpty( ) ); }

    bool  bSetBinary ( const wxString & rosFName  );
    bool  bSetArgLst ( const wxString & rosArgLst );
    bool  bSetLogFile( const wxString & rosFName  );

    const wxFileName & roGetBinary ( void ) { return( m_oFnBinary         ); }
    const wxChar     * psGetArgLst ( void ) { return( m_osArgLst.c_str( ) ); }
    const wxFileName & roGetLogFile( void ) { return( m_oFnLog            ); }
    const wxChar     * psGetErrMsg ( void ) { return( m_osErrMsg.c_str( ) ); }

    bool  bDelLogFile( void );

    bool  bExec( const wxChar * psArgLst=NULL );
    bool  bKill( void );

    void  OnTerminate( int iPid, int iStatus );

    void  Print( TextCtrl & roTxtCtl );
};

//*****************************************************************************

#endif // PRCBASE_HPP
